@@ -289,55 +289,53 @@ typedef struct {
   SCM arg2;
   SCM arg3;
   SCM retval;
+  gboolean exception;
 } CallData;
 
-#define CALL_DATA_INIT  { 0, 0, 0, 0, 0, 0 };
+#define CALL_DATA_INIT  { 0, 0, 0, 0, 0, 0, 0 };
 
-static void
-cscmi_write_exception_details (int error_fd, SCM tag, SCM throw_args)
+static char *
+cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
 {
   AisleriotGame *game = app_game;
-  char *message;
   SCM port;
   SCM stack;
   GPtrArray *slots;
   guint i, n_slots;
+  GString *message;
+  char *string;
 
-  message = g_strdup_printf ("Variation: %s\n", aisleriot_game_get_game_file (game));
+  message = g_string_sized_new (1024);
 
-  write (error_fd, message, strlen (message));
-  g_free (message);
+  g_string_append_printf (message, "Variation: %s\n", aisleriot_game_get_game_file (game));
+  g_string_append_printf (message, "Seed: %u\n", game->seed);
 
-  message = g_strdup_printf ("Seed: %u\n", game->seed);
-  write (error_fd, message, strlen (message));
-  g_free (message);
+  port = scm_open_output_string ();
 
-  message = "Scheme error:\n\t";
-  write (error_fd, message, strlen (message));
-
-  port = scm_fdopen (scm_from_int (error_fd),
-                     scm_mem2string ("w", sizeof (char)));
+  g_string_append (message, "Scheme error:\n\t");
   scm_display (throw_args, port);
-  scm_fsync (port);
+  string = scm_to_locale_string (scm_get_output_string (port));
+  g_string_append (message, string);
+  free (string);
 
-  message = "\nScheme tag:\n\t";
-  write (error_fd, message, strlen (message));
+  g_string_append (message, "\nScheme tag:\n\t");
   scm_display (tag, port);
-  scm_fsync (port);
+  string = scm_to_locale_string (scm_get_output_string (port));
+  g_string_append (message, string);
+  free (string);
 
-  message = "\n\nBacktrace:\n";
-  write (error_fd, message, strlen (message));
+  g_string_append (message, "\n\nBacktrace:\n");
   stack = scm_fluid_ref (SCM_VARIABLE_REF (scm_the_last_stack_fluid_var));
   if (!SCM_FALSEP (stack)) {
     scm_display_backtrace (stack, port, SCM_UNDEFINED, SCM_UNDEFINED);
-    scm_fsync (port);
+    string = scm_to_locale_string (scm_get_output_string (port));
+    g_string_append (message, string);
+    free (string);
   } else {
-    message = "\tNo backtrace available.\n";
-    write (error_fd, message, strlen (message));
+    g_string_append (message, "\tNo backtrace available.\n");
   }
 
-  message = "\n\nDeck State:\n";
-  write (error_fd, message, strlen (message));
+  g_string_append (message, "\n\nDeck State:\n");
 
   slots = aisleriot_game_get_slots (game);
 
@@ -348,9 +346,7 @@ cscmi_write_exception_details (int error_fd, SCM tag, SCM throw_args)
       GByteArray *cards = slot->cards;
       guint n_cards;
 
-      message = g_strdup_printf ("\tSlot %d\n", slot->id);
-      write (error_fd, message, strlen (message));
-      g_free (message);
+      g_string_append_printf (message, "\tSlot %d\n", slot->id);
 
       n_cards = cards->len;
       if (n_cards > 0) {
@@ -360,61 +356,72 @@ cscmi_write_exception_details (int error_fd, SCM tag, SCM throw_args)
         for (count = 0; count < n_cards; ++count) {
           Card card = CARD (data[count]);
 
-          if (count == 0)
-            message = "\t\t";
-          else
-            message = ", ";
-          write (error_fd, message, strlen (message));
+          if (count == 0) {
+            g_string_append_c (message, '\t');
+            g_string_append_c (message, '\t');
+          } else {
+            g_string_append_c (message, ' ');
+            g_string_append_c (message, ',');
+          }
+
+          g_string_append_printf (message,
+                                  "(%d %d %s)",
+                                  CARD_GET_SUIT (card),
+                                  CARD_GET_RANK (card),
+                                  /* See c2scm_card below */
+                                  CARD_GET_FACE_DOWN (card) ? "#f" : "#t");
 
-          message = g_strdup_printf ("(%d %d %s)",
-                                     CARD_GET_SUIT (card),
-                                     CARD_GET_RANK (card),
-                                     /* See c2scm_card below */
-                                     CARD_GET_FACE_DOWN (card) ? "#f" : "#t");
-          write (error_fd, message, strlen (message));
-          g_free (message);
           count++;
           if (count == 5) {
-            message = "\n";
-            write (error_fd, message, strlen (message));
+            g_string_append_c (message, '\n');
             count = 0;
           }
         }
         if (count != 0) {
-          message = "\n";
-          write (error_fd, message, strlen (message));
+          g_string_append_c (message, '\n');
         }
       } else {
-        message = "\t\t(Empty)\n";
-        write (error_fd, message, strlen (message));
+        g_string_append (message,"\t\t(Empty)\n");
       }
     }
   } else {
-    message = "\tNo cards in deck\n";
-    write (error_fd, message, strlen (message));
+    g_string_append (message, "\tNo cards in deck\n");
   }
+
+  return g_string_free (message, FALSE);
 }
 
 /* Called when we get an exception from guile.  We launch bug-buddy with the
  * exception information:
  */
 static SCM
-cscmi_catch_handler (gpointer data G_GNUC_UNUSED,
+cscmi_catch_handler (gpointer user_data,
                      SCM tag,
                      SCM throw_args)
 {
+  CallData *data = (CallData *) user_data;
   AisleriotGame *game = app_game;
+  char *message;
   int error_fd;
   char *error_file = NULL;
   GError *error = NULL;
 
+  if (data)
+    data->exception = TRUE;
+
   if (game->had_exception)
     goto out;
 
+  message = cscmi_exception_get_backtrace (tag, throw_args);
+  if (!message) {
+    g_warning ("A scheme exception occurred, but there was no exception info\n");
+    goto out;
+  }
+
   error_fd = g_file_open_tmp ("arcrashXXXXXX", &error_file, &error);
   if (error_fd >= 0) {
-    cscmi_write_exception_details (error_fd, tag, throw_args);
     close (error_fd);
+    g_file_set_contents (error_file, message, strlen (message), NULL);
 
     /* Tell the frontend about the problem */
     g_signal_emit (game, signals[EXCEPTION], 0, error_file);
@@ -424,11 +431,13 @@ cscmi_catch_handler (gpointer data G_GNUC_UNUSED,
                error->message);
     g_error_free (error);
   }
-  
+
 out:
   /* This game is over, but don't count it in the statistics */
   set_game_state (game, GAME_LOADED);
 
+  g_free (message);
+
   return SCM_UNDEFINED;
 }
 
@@ -462,9 +471,16 @@ cscmi_call_lambda (void *user_data)
 static SCM
 cscmi_eval_string (const char *string)
 {
-  return scm_internal_stack_catch (SCM_BOOL_T,
-                                   (scm_t_catch_body) scm_c_eval_string, (void *) string,
-                                   cscmi_catch_handler, NULL);
+  CallData data = CALL_DATA_INIT;
+  SCM retval;
+
+  retval = scm_internal_stack_catch (SCM_BOOL_T,
+                                     (scm_t_catch_body) scm_c_eval_string, (void *) string,
+                                     cscmi_catch_handler, &data);
+  if (data.exception)
+    return SCM_EOL;
+
+  return retval;
 }
 
 static SCM
@@ -943,7 +959,10 @@ scm_execute_delayed_function (SCM callback)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+
+  if (data.exception)
+    return FALSE;
 
   aisleriot_game_test_end_of_game (game);
 
@@ -1024,12 +1043,14 @@ update_game_dealable (AisleriotGame *game)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return;
 
   set_game_dealable (game, SCM_NFALSEP (data.retval));
 }
 
-static void
+static gboolean
 cscmi_start_game_lambda (double *width,
                          double *height)
 {
@@ -1040,10 +1061,13 @@ cscmi_start_game_lambda (double *width,
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   *width = scm_num2double (SCM_CAR (data.retval), 0, NULL);
   *height = scm_num2double (SCM_CADR (data.retval), 0, NULL);
+  return TRUE;
 }
 
 static gboolean
@@ -1056,7 +1080,9 @@ cscmi_game_over_lambda (void)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return TRUE;
 
   return SCM_NFALSEP (data.retval);
 }
@@ -1071,7 +1097,9 @@ cscmi_winning_game_lambda (void)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   return SCM_NFALSEP (data.retval);
 }
@@ -1629,7 +1657,7 @@ aisleriot_game_new_game (AisleriotGame *game,
     /* FIXMEchpe: We should NOT re-seed here, but just get another random number! */
     g_random_set_seed (game->seed);
 
-    cscmi_start_game_lambda (&game->width, &game->height);
+    cscmi_start_game_lambda (&game->width, &game->height); /* FIXME this may fail */
     scm_c_eval_string ("(start-game)");
   } while (!cscmi_game_over_lambda ());
 
@@ -1737,9 +1765,12 @@ aisleriot_game_drag_valid (AisleriotGame *game,
   scm_gc_protect_object (data.arg2);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
   scm_gc_unprotect_object (data.arg2);
 
+  if (data.exception)
+    return FALSE;
+
   return SCM_NFALSEP (data.retval);
 }
 
@@ -1773,9 +1804,12 @@ aisleriot_game_drop_valid (AisleriotGame *game,
   data.arg3 = scm_from_int (end_slot);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
   scm_gc_unprotect_object (data.arg2);
 
+  if (data.exception)
+    return FALSE;
+
   return SCM_NFALSEP (data.retval);
 }
 
@@ -1805,9 +1839,12 @@ aisleriot_game_drop_cards (AisleriotGame *game,
   data.arg3 = scm_from_int (end_slot);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
   scm_gc_unprotect_object (data.arg2);
 
+  if (data.exception)
+    return FALSE;
+
   return SCM_NFALSEP (data.retval);
 }
 
@@ -1831,7 +1868,9 @@ aisleriot_game_button_clicked_lambda (AisleriotGame *game,
   data.arg1 = scm_from_int (slot_id);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   return SCM_NFALSEP (data.retval);
 }
@@ -1856,75 +1895,291 @@ aisleriot_game_button_double_clicked_lambda (AisleriotGame *game,
   data.arg1 = scm_from_int (slot_id);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   return SCM_NFALSEP (data.retval);
 }
 
 /**
- * aisleriot_game_hint_lambda:
+ * aisleriot_game_get_hint:
  * @game:
  *
  * Gets a hint.
  *
- * Returns: a #SCM containing the hint
+ * Returns: a newly allocated string containing the hint message
  */
-SCM
-aisleriot_game_hint_lambda (AisleriotGame *game)
+char *
+aisleriot_game_get_hint (AisleriotGame *game)
 {
   CallData data = CALL_DATA_INIT;
+  SCM hint, string1, string2;
+  char *message = NULL;
+  char *str1, *str2;
 
   data.lambda = game->hint_lambda;
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return NULL;
 
-  return data.retval;
+  hint = data.retval;
+
+  if (!SCM_NFALSEP (hint)) {
+    message = g_strdup (_("This game does not have hint support yet."));
+  } else {
+    switch (scm_num2int (SCM_CAR (hint), SCM_ARG1, NULL)) {
+
+    case 0:
+      string1 = SCM_CADR (hint);
+      if (!scm_is_string (string1))
+        break;
+
+      str1 = scm_to_locale_string (string1);
+      if (!str1)
+        break;
+
+      message = g_strdup (str1);
+      free (str1);
+      break;
+
+    case 1:
+      string1 = SCM_CADR (hint);
+      string2 = SCM_CADDR (hint);
+
+      if (!scm_is_string (string1) || !scm_is_string (string2))
+        break;
+
+      str1 = scm_to_locale_string (string1);
+      if (!str1)
+        break;
+      str2 = scm_to_locale_string (string2);
+      if (!str2) {
+        free (str1);
+        break;
+      }
+
+      /* Both %s are card names */
+      message = g_strdup_printf (_("Move %s onto %s."), str1, str2);
+      free (str1);
+      free (str2);
+      break;
+
+    case 2:
+      /* NOTE! This case is exactly the same as case 1, but the strings
+        * are different: the first is a card name, the 2nd a sentence fragment.
+        * NOTE! FIXMEchpe! This is bad for i18n.
+        */
+      string1 = SCM_CADR (hint);
+      string2 = SCM_CADDR (hint);
+
+      if (!scm_is_string (string1) || !scm_is_string (string2))
+        break;
+
+      str1 = scm_to_locale_string (string1);
+      if (!str1)
+        break;
+      str2 = scm_to_locale_string (string2);
+      if (!str2) {
+        free (str1);
+        break;
+      }
+
+      /* The first %s is a card name, the 2nd %s a sentence fragment.
+        * Yes, we know this is bad for i18n.
+        */
+      message = g_strdup_printf (_("Move %s onto %s."), str1, str2);
+      free (str1);
+      free (str2);
+      break;
+
+    case 3: /* This is deprecated (due to i18n issues) do not use. */
+      g_warning ("This game uses a deprecated hint method (case 3).\n"
+                 "Please file a bug at http://bugzilla.gnome.org "
+                 "including this message and the name of the game "
+                 "you were playing, which is %s.\n",
+                 aisleriot_game_get_game_file (game));
+      break;
+
+    case 4:
+      string1 = SCM_CADR (hint);
+      if (!scm_is_string (string1))
+        break;
+
+      str1 = scm_to_locale_string (string1);
+      if (!str1)
+        break;
+
+      message = g_strdup_printf (_("You are searching for a %s."), str1);
+      free (str1);
+      break;
+
+    default:
+      message = g_strdup (_("This game is unable to provide a hint."));
+      break;
+    }
+  }
+
+  return message;
 }
 
 /**
- * aisleriot_game_get_options_lambda:
+ * aisleriot_game_option_free:
+ * @option: a #AisleriotGameOption
+ *
+ * Frees @options.
+ */
+void
+aisleriot_game_option_free (AisleriotGameOption *option)
+{
+  g_return_if_fail (option != NULL);
+
+  g_free (option->display_name);
+  g_slice_free (AisleriotGameOption, option);
+}
+
+/**
+ * aisleriot_game_get_options:
  * @game:
  *
- * Returns: a #SCM containing the game options
+ * Returns: a newly allocated list containing newly allocated
+ * #AisleriotGameOption structs. 
  */
-SCM
-aisleriot_game_get_options_lambda (AisleriotGame *game)
+GList *
+aisleriot_game_get_options (AisleriotGame *game)
 {
   CallData data = CALL_DATA_INIT;
+  SCM options_list;
+  int l, i;
+  guint32 bit = 1;
+  AisleriotGameOptionType type = AISLERIOT_GAME_OPTION_CHECK;
+  GList *options = NULL;
 
   data.lambda = game->get_options_lambda;
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
-  return data.retval;
+  options_list = data.retval;
+  if (scm_is_false (scm_list_p (options_list)))
+    return NULL;
+
+  l = scm_to_int (scm_length (options_list));
+  bit = 1;
+  for (i = 0; i < l; i++) {
+    SCM entry;
+
+    /* Each entry in the options list is a list consisting of a name and
+     * a variable.
+     */
+    entry = scm_list_ref (options_list, scm_from_int (i));
+    if (!scm_is_false (scm_list_p (entry))) {
+      SCM entryname;
+      char *entrynamestr;
+      gboolean entrystate;
+      AisleriotGameOption *option;
+
+      entryname = scm_list_ref (entry, scm_from_uint (0));
+      if (!scm_is_string (entryname))
+        continue; /* Shouldn't happen */
+
+      entrynamestr = scm_to_locale_string (entryname);
+      if (!entrynamestr)
+        continue;
+
+      entrystate = SCM_NFALSEP (scm_list_ref (entry, scm_from_uint (1)));
+
+      option = g_slice_new (AisleriotGameOption);
+      option->display_name = g_strdup (entrynamestr);
+      option->type = type;
+      option->value = bit;
+      option->set = entrystate != FALSE;
+
+      options = g_list_prepend (options, option);
+
+      free (entrynamestr);
+
+      bit <<= 1;
+    } else {
+      /* If we encounter an atom, change the mode. What the atom is doesn't
+      * really matter. */
+      if (type == AISLERIOT_GAME_OPTION_CHECK) {
+        type = AISLERIOT_GAME_OPTION_RADIO;
+      } else {
+        type = AISLERIOT_GAME_OPTION_CHECK;
+      }
+    }
+  }
+
+  return g_list_reverse (options);
 }
 
 /**
  * aisleriot_game_apply_options_lambda:
  * @game:
- * @options:
+ * @changed_mask:
+ * @changed_value:
  *
- * Applies @options.
+ * Applies options.
  *
- * Returns: a #SCM with status information
+ * Returns: the new options value
  */
-SCM
-aisleriot_game_apply_options_lambda (AisleriotGame *game, SCM options)
+guint32
+aisleriot_game_change_options (AisleriotGame *game,
+                               guint32 changed_mask,
+                               guint32 changed_value)
 {
   CallData data = CALL_DATA_INIT;
+  CallData data2 = CALL_DATA_INIT;
+  SCM options_list;
+  guint32 bit, value;
+  int l, i;
 
-  data.lambda = game->apply_options_lambda;
-  data.n_args = 1;
-  data.arg1 = options;
+  data.lambda = game->get_options_lambda;
+  data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return 0;
 
-  return data.retval;
+  options_list = data.retval;
+  if (scm_is_false (scm_list_p (options_list)))
+    return 0;
+
+  value = 0;
+  bit = 1;
+  l = scm_to_int (scm_length (options_list));
+  for (i = 0; i < l; i++) {
+    SCM entry;
+
+    entry = scm_list_ref (options_list, scm_from_uint (i));
+    if (scm_is_false (scm_list_p (entry)))
+      continue;
+
+    if (changed_mask & bit)
+      scm_list_set_x (entry, scm_from_uint (1), (changed_value & bit) ? SCM_BOOL_T : SCM_BOOL_F);
+  
+    if (SCM_NFALSEP (scm_list_ref (entry, scm_from_uint (1))))
+      value |= bit;
+
+    bit <<= 1;
+  }
+
+  data2.lambda = game->apply_options_lambda;
+  data2.n_args = 1;
+  data2.arg1 = options_list;
+  scm_internal_stack_catch (SCM_BOOL_T,
+                            cscmi_call_lambda, &data2,
+                            cscmi_catch_handler, &data);
+
+  return value;
 }
 
 /**
@@ -1944,7 +2199,9 @@ aisleriot_game_timeout_lambda (AisleriotGame *game)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   return SCM_NFALSEP (data.retval);
 }
