@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 __author__ = 'Robert Ancell <bob27@users.sourceforge.net>'
 __license__ = 'GNU General Public License Version 2'
 __copyright__ = 'Copyright 2005-2006  Robert Ancell'
@@ -454,7 +455,7 @@ class Player(game.ChessPlayer):
                 
                     # Bridge information between the application and the engine
                     os.write(targets[fd], data)
-        except:
+        except Exception, e:
             # Kill the child and exit
             try:
                 os.kill(enginePID, signal.SIGQUIT)
