@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import protocol
 import xml.sax.saxutils
 
@@ -35,7 +36,7 @@ class ChannelFeedback:
     
 class ClientFeedback:
     
-    def setBusy(self):
+    def setBusy(self, isBusy):
         """Called when the client is busy (unable to take requests)"""
         pass
     
@@ -349,7 +350,12 @@ class MainChannel(ChannelFeedback, protocol.ParserFeedback):
         self.client.feedback.tableUpdated(table)
 
     def tableRemoved(self, tableId):
-        table = self.client.tables.pop(tableId)
+        try:
+            table = self.client.tables.pop(tableId)
+        except KeyError:
+            # We do not know of this table - this could occur if we receive a
+            # table remove event before we get the table list.
+            return
         self.client.feedback.tableRemoved(table)
         
     def onPlayerList(self, room, players):
@@ -397,7 +403,13 @@ class MainChannel(ChannelFeedback, protocol.ParserFeedback):
         self.client.feedback.playerAdded(player)
 
     def playerRemoved(self, name, room, toRoom):
-        player = self.client.players.pop(name)
+        try:
+            player = self.client.players.pop(name)
+        except KeyError:
+            # We do not know of this player - this could occur if we receive a
+            # player remove event before we get the player list.
+            return
+            
         player.room.nPlayers -= 1
         player.lastRoom = player.room
         try:
@@ -463,7 +475,7 @@ class GameChannel(ChannelFeedback, protocol.ParserFeedback):
         self.inSession = False
         self.client.mainChannel.send(self.command)
 
-    def closed(self):
+    def closed(self, errno = 0):
         print 'SEVERE: GGZ channel closed'
 
 class Client:
