@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #!/usr/bin/env python
 # png.py - PNG encoder in pure Python
 # Copyright (C) 2006 Johann C. Rocholl <johann@browsershots.org>
@@ -50,9 +51,9 @@ prompt for usage and a list of options.
 """
 
 
-__revision__ = '$Rev$'
-__date__ = '$Date$'
-__author__ = '$Author$'
+__revision__ = '$Rev: 8277 $'
+__date__ = '$Date: 2008-11-08 03:48:10 +0100 (Sat, 08 Nov 2008) $'
+__author__ = '$Author: rancell $'
 
 
 import sys
@@ -209,7 +210,7 @@ class Writer:
         outfile.write(data)
         checksum = zlib.crc32(tag)
         checksum = zlib.crc32(data, checksum)
-        outfile.write(struct.pack("!I", checksum))
+        outfile.write(struct.pack("!i", checksum))
 
     def write(self, outfile, scanlines):
         """
@@ -403,10 +404,10 @@ class _readable:
         self.offset = 0
 
     def read(self, n):
-        r = buf[offset:offset+n]
+        r = self.buf[self.offset:self.offset+n]
         if isinstance(r, array):
             r = r.tostring()
-        offset += n
+        self.offset += n
         return r
 
 class Reader:
@@ -468,8 +469,8 @@ class Reader:
         verify = struct.pack('!i', verify)
         if checksum != verify:
             # print repr(checksum)
-            (a,) = struct.unpack('!I', checksum)
-            (b,) = struct.unpack('!I', verify)
+            (a, ) = struct.unpack('!I', checksum)
+            (b, ) = struct.unpack('!I', verify)
             raise ValueError("Checksum error in %s chunk: 0x%X != 0x%X"
                              % (tag, a, b))
         return tag, data
@@ -699,7 +700,7 @@ class Reader:
             try:
                 tag, data = self.read_chunk()
             except ValueError, e:
-                raise Error('Chunk error: ' + e.args[0])
+                raise Error('Chunk error: ' + e.message)
 
             # print >> sys.stderr, tag, len(data)
             if tag == 'IHDR': # http://www.w3.org/TR/PNG/#11IHDR
