@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 
 class StateMachine:
     """
@@ -8,6 +9,9 @@ class StateMachine:
     
     options = None
     
+    __positionCommand = 'position startpos'
+    __haveMoves = False
+    
     buffer = ''
     
     __readyToConfigure = False
@@ -58,7 +62,7 @@ class StateMachine:
             options = self.__options
             self.__options = None
             self.configure(options)
-            
+
         # Send queued commands once have OK
         if len(self.__queuedCommands) > 0 and self.__ready:
             commands = self.__queuedCommands
@@ -79,7 +83,13 @@ class StateMachine:
         """
         self.onOutgoingData('uci\n')
         
-    def configure(self, options):
+    def startGame(self):
+        """
+        """
+        self.__sendCommand('ucinewgame')
+        self.__sendCommand(self.__positionCommand)
+
+    def configure(self, options = []):
         """
         """
         if not self.__readyToConfigure:
@@ -95,15 +105,26 @@ class StateMachine:
             self.onOutgoingData('setoption ' + option.name + ' value ' + option.value + '\n')
         self.onOutgoingData('isready\n')
 
-    def requestMove(self):
+    def requestMove(self, whiteTime, blackTime, ownTime):
         """
         """
-        self.__sendCommand('go')
+        # Some AI's don't work unless assigned some time
+        # TODO: which ones? I think Glaurung had issues
+        if whiteTime == 0:
+            whiteTime = 30000
+        if blackTime == 0:
+            blackTime = 30000
+            
+        self.__sendCommand('go wtime %d btime %d' % (whiteTime, blackTime))
 
     def reportMove(self, move, isSelf):
         """
         """
-        self.__sendCommand('position moves ' + move)
+        if not self.__haveMoves:
+            self.__positionCommand += ' moves'
+        self.__haveMoves = True
+        self.__positionCommand += ' ' + move
+        self.__sendCommand(self.__positionCommand)
         
     def parseLine(self, line):
         """
@@ -139,8 +160,6 @@ class StateMachine:
             if len(args) != 0:
                 print 'WARNING: Arguments on readyok: ' + str(args)
             self.__ready = True
-            self.__sendCommand('ucinewgame')
-            self.__sendCommand('position startpos')
             return 'info'
         
         elif command == 'bestmove':
