@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 """
 
@@ -125,6 +126,9 @@ class ViewFeedback:
         'showHints' sets if move hints should be shown (boolean).
         """
         assert(False)
+        
+    def showSmooth(self, doNumbering):
+        pass
     
     def saveGame(self, path):
         """Called when the user requests the game in this view to be saved.
@@ -406,7 +410,10 @@ class UI:
         When data is available onReadFileDescriptor() is called.
         """
         assert(False)
-    
+
+    def unwatchFileDescriptor(self, fd):
+        assert(False)
+        
     def writeFileDescriptor(self, fd):
         """Notify when a file descriptor can be written to.
         
