@@ -20,14 +20,18 @@
  */
 
 #include <config.h>
+
 #include <string.h>
-#include <gnome.h>
+#include <stdlib.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #include "gnibbles.h"
 #include "scoreboard.h"
 
 GnibblesScoreboard *
-gnibbles_scoreboard_new (GtkWidget * t_appbar)
+gnibbles_scoreboard_new (GtkWidget * t_statusbar)
 {
   int i;
   char buffer[255];
@@ -39,20 +43,18 @@ gnibbles_scoreboard_new (GtkWidget * t_appbar)
   tmp->count = 0;
 
   for (i = 0; i < NUMWORMS; i++) {
-    hbox = gtk_hbox_new (FALSE, GNOME_PAD);
+    hbox = gtk_hbox_new (FALSE, 8);
     gtk_widget_show (hbox);
 
     sprintf (buffer, _("Worm %d:"), i + 1);
     tmp->names[i] = gtk_label_new (buffer);
     gtk_widget_set_sensitive (tmp->names[i], FALSE);
-    gtk_widget_show (tmp->names[i]);
     gtk_box_pack_start (GTK_BOX (hbox), tmp->names[i], FALSE, FALSE, 0);
-    tmp->data[i] = gtk_label_new ("00, 00000");
+    tmp->data[i] = gtk_label_new ("00, 0000");
     gtk_widget_set_sensitive (tmp->data[i], FALSE);
-    gtk_widget_show (tmp->data[i]);
     gtk_box_pack_start (GTK_BOX (hbox), tmp->data[i], FALSE, FALSE, 0);
 
-    gtk_box_pack_start (GTK_BOX (t_appbar), hbox, FALSE, FALSE, GNOME_PAD);
+    gtk_box_pack_start (GTK_BOX (t_statusbar), hbox, FALSE, FALSE, 4);
   }
 
   return (tmp);
@@ -60,11 +62,18 @@ gnibbles_scoreboard_new (GtkWidget * t_appbar)
 
 void
 gnibbles_scoreboard_register (GnibblesScoreboard * scoreboard,
-			      GnibblesWorm * t_worm)
+			      GnibblesWorm * t_worm, gchar * colorname)
 {
+  GdkColor color;
+
+  gdk_color_parse (colorname, &color);
+
   scoreboard->worms[scoreboard->count] = t_worm;
   gtk_widget_set_sensitive (scoreboard->names[scoreboard->count], TRUE);
+  gtk_widget_modify_fg (scoreboard->names[scoreboard->count], GTK_STATE_NORMAL, &color); 
   gtk_widget_set_sensitive (scoreboard->data[scoreboard->count], TRUE);
+  gtk_widget_show (scoreboard->names[scoreboard->count]);
+  gtk_widget_show (scoreboard->data[scoreboard->count]);
   scoreboard->count++;
 }
 
@@ -76,7 +85,7 @@ gnibbles_scoreboard_update (GnibblesScoreboard * scoreboard)
   const gchar *buffer2;
 
   for (i = 0; i < scoreboard->count; i++) {
-    buffer = g_strdup_printf ("%02d, %05d",
+    buffer = g_strdup_printf ("%02d, %04d",
 			      (scoreboard->worms[i]->lives > -1) ?
 			      scoreboard->worms[i]->lives : 0,
 			      scoreboard->worms[i]->score);
@@ -95,7 +104,7 @@ gnibbles_scoreboard_clear (GnibblesScoreboard * scoreboard)
   scoreboard->count = 0;
 
   for (i = 0; i < NUMWORMS; i++) {
-    gtk_widget_set_sensitive (scoreboard->names[i], FALSE);
-    gtk_widget_set_sensitive (scoreboard->data[i], FALSE);
+    gtk_widget_hide (scoreboard->names[i]);
+    gtk_widget_hide (scoreboard->data[i]);
   }
 }
