@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import gtk, cairo, pango, gobject
 import colors
 import math
@@ -119,6 +120,7 @@ class NumberBox (gtk.Widget):
     top_note_text = ''
     bottom_note_text = ''    
     read_only = False
+    read_only_hidden = False
     _layout = None
     _top_note_layout = None
     _bottom_note_layout = None    
@@ -1258,7 +1260,8 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
             x,y,v = self.trackers[identifier][0]
             ret.append((x,y,v,self.trackers_for_point(x,y,v)))
             self.remove(x,y)
-            self.grid.remove(x,y)
+            if self.grid and self.grid._get_(x,y):
+                self.grid.remove(x,y)
         return ret
 
     def delete_except_for_tracker (self, identifier):
@@ -1273,7 +1276,9 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
                     ):
                     removed.append((x,y,val,self.trackers_for_point(x,y,val)))
                     self.remove(x,y)
-                    self.grid.remove(x,y)
+                    if self.grid and self.grid._get_(x,y):
+                        self.grid.remove(x,y)
+
         return removed
 
     def add_tracker (self, x, y, tracker, val=None):
@@ -1316,7 +1321,9 @@ class GridDancer:
         self.grid = grid
         self.dancing = False
         for box in self.grid.__entries__.values():
-            box.read_only = False
+            if box.read_only == True:
+                box.read_only = False
+                box.read_only_hidden = True
             box.queue_draw()
 
     def start_dancing (self):
@@ -1330,6 +1337,9 @@ class GridDancer:
         self.dancing = False
         for box in self.grid.__entries__.values():
             box.props.can_focus = True
+            if box.read_only_hidden == True:
+                box.read_only = True
+                box.read_only_hidden = False
         self.grid.unhighlight_cells()
 
     def do_dance_step (self):
