@@ -1,3 +1,4 @@
+/* -*- mode:C++; tab-width:8; c-basic-offset:8; indent-tabs-mode:true -*- */
 /*
  * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
@@ -25,6 +26,10 @@
 #include <libgames-support/games-conf.h>
 #include <libgames-support/games-runtime.h>
 
+#ifdef HAVE_CLUTTER
+#include <clutter-gtk/gtk-clutter-embed.h>
+#endif
+
 #include "tetris.h"
 
 int
@@ -55,6 +60,9 @@ main(int argc, char *argv[])
 	GOptionContext *context = g_option_context_new (NULL);
 
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+#ifdef HAVE_CLUTTER
+	g_option_context_add_group (context, clutter_get_option_group_without_init ());
+#endif
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
 	games_sound_add_option_group (context);
 
@@ -66,12 +74,20 @@ main(int argc, char *argv[])
 		return 1;
 	}
 
-	g_set_application_name (_("Five or More"));
+	g_set_application_name (_("Gnometris"));
 
 	gtk_window_set_default_icon_name ("gnome-gnometris");
 
 	games_conf_initialise ("Gnometris");
 
+#ifdef HAVE_CLUTTER
+	if (gtk_clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS ) {
+		g_printerr ("Failed to initialise clutter: %s\n", error->message);
+		g_error_free (error);
+		return 1;
+	}
+#endif
+
 	Tetris *t = new Tetris(cmdlineLevel);
 
 	gtk_main();
