@@ -30,10 +30,11 @@
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #include <config.h>
-#include <gnome.h>
 
-#include <games-frame.h>
-#include <games-conf.h>
+#include <gtk/gtk.h>
+
+#include <libgames-support/games-frame.h>
+#include <libgames-support/games-conf.h>
 
 #include "yahtzee.h"
 #include "gyahtzee.h"
