@@ -22,21 +22,31 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+
+#include <string.h>
+#include <stdlib.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include <string.h>
-#include <games-stock.h>
-#include <games-sound.h>
-#include <games-conf.h>
+#include <libgames-support/games-conf.h>
+#include <libgames-support/games-help.h>
+#include <libgames-support/games-runtime.h>
+#include <libgames-support/games-sound.h>
+#include <libgames-support/games-stock.h>
+
+#ifdef WITH_SMCLIENT
+#include <libgames-support/eggsmclient.h>
+#endif /* WITH_SMCLIENT */
 
 #ifdef GGZ_CLIENT
 #include <games-dlg-chat.h>
 #include <games-dlg-players.h>
 #include "ggz-network.h"
 #include <ggz-embed.h>
-#endif
+#endif /* WITH GGZ_CLIENT */
 
 #include "gnothello.h"
 #include "othello.h"
@@ -45,20 +55,27 @@
 #define APP_NAME "iagno"
 #define APP_NAME_LONG N_("Iagno")
 
-GnomeAppBar *appbar;
 GtkWidget *window;
+GtkWidget *statusbar;
 GtkWidget *notebook;
 GtkWidget *drawing_area;
 GtkWidget *tile_dialog;
 GtkWidget *black_score;
 GtkWidget *white_score;
 
+GtkAction *new_game_action;
+GtkAction *new_network_action;
+GtkAction *leave_network_action;
+GtkAction *player_list_action;
+GtkAction *undo_action;
+
 GdkPixmap *buffer_pixmap = NULL;
 GdkPixmap *tiles_pixmap = NULL;
 GdkPixmap *tiles_mask = NULL;
 
-gint flip_pixmaps_id = 0;
-gint statusbar_id;
+static gint flip_pixmaps_id = 0;
+static gint flip_animation_speed = PIXMAP_FLIP_DELAY;
+guint statusbar_id;
 guint black_computer_level;
 guint white_computer_level;
 guint black_computer_id = 0;
@@ -133,50 +150,10 @@ static const GOptionEntry options[] = {
   {NULL}
 };
 
-GnomeUIInfo game_menu[] = {
-  GNOMEUIINFO_MENU_NEW_GAME_ITEM (new_game_cb, NULL),
-
-  GNOMEUIINFO_ITEM (N_("Net_work Game"), NULL, new_network_game_cb, NULL),
-
-  GNOMEUIINFO_SEPARATOR,
-
-  GNOMEUIINFO_MENU_UNDO_MOVE_ITEM (undo_move_cb, NULL),
-
-  GNOMEUIINFO_ITEM (N_("_Player list"), NULL, on_player_list, NULL),
-
-  GNOMEUIINFO_ITEM (N_("_Chat Window"), NULL, on_chat_window, NULL),
-
-  GNOMEUIINFO_SEPARATOR,
-
-  GNOMEUIINFO_ITEM (N_("_Leave Game"), NULL, on_network_leave, NULL),
-
-  GNOMEUIINFO_MENU_QUIT_ITEM (quit_game_cb, NULL),
-
-  GNOMEUIINFO_END
-};
-
-GnomeUIInfo settings_menu[] = {
-  GNOMEUIINFO_MENU_PREFERENCES_ITEM (properties_cb, NULL),
-  GNOMEUIINFO_END
-};
-
-GnomeUIInfo help_menu[] = {
-  GNOMEUIINFO_HELP ("iagno"),
-  GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
-  GNOMEUIINFO_END
-};
-
-GnomeUIInfo mainmenu[] = {
-  GNOMEUIINFO_MENU_GAME_TREE (game_menu),
-  GNOMEUIINFO_MENU_SETTINGS_TREE (settings_menu),
-  GNOMEUIINFO_MENU_HELP_TREE (help_menu),
-  GNOMEUIINFO_END
-};
-
 static void
 undo_set_sensitive (gboolean state)
 {
-  gtk_widget_set_sensitive (game_menu[3].widget, state);
+  gtk_action_set_sensitive (undo_action, state);
 }
 
 void
@@ -196,7 +173,7 @@ on_player_list (void)
 {
 #ifdef GGZ_CLIENT
   create_or_raise_dlg_players (GTK_WINDOW (window));
-#endif
+#endif /* GGZ_CLIENT */
 }
 
 void
@@ -204,7 +181,7 @@ on_chat_window (void)
 {
 #ifdef GGZ_CLIENT
   create_or_raise_dlg_chat (GTK_WINDOW (window));
-#endif
+#endif /* GGZ_CLIENT */
 }
 
 void
@@ -212,9 +189,10 @@ new_network_game_cb (GtkWidget * widget, gpointer data)
 {
 #ifdef GGZ_CLIENT
   on_network_game ();
-  gtk_widget_hide (mainmenu[0].widget);
-  gtk_widget_hide (mainmenu[1].widget);
-#endif
+  gtk_action_set_sensitive (new_network_action, TRUE);
+  gtk_action_set_sensitive (leave_network_action, TRUE);
+  gtk_action_set_sensitive (player_list_action, TRUE);
+#endif /* GGZ_CLIENT */
 }
 
 void
@@ -223,7 +201,7 @@ on_network_leave (GObject * object, gpointer data)
 #ifdef GGZ_CLIENT
   ggz_embed_leave_table ();
   gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), NETWORK_PAGE);
-#endif
+#endif /* GGZ_CLIENT */
 }
 
 void
@@ -266,12 +244,12 @@ undo_move_cb (GtkWidget * widget, gpointer data)
   }
 
   gui_status ();
-  tiles_to_flip = 1;
+  start_animation ();
   check_computer_players ();
 }
 
 void
-about_cb (GtkWidget * widget, gpointer data)
+about_cb (GtkAction * action, gpointer data)
 {
   const gchar *authors[] = { "Ian Peters", NULL };
 
@@ -362,7 +340,7 @@ button_press_event (GtkWidget * widget, GdkEventButton * event)
 	&& is_valid_move (x, y, whose_turn)) {
 #ifdef GGZ_CLIENT
       send_my_move (CART (x + 1, y + 1), whose_turn);
-#endif
+#endif /* GGZ_CLIENT */
     } else if (!ggz_network_mode && is_valid_move (x, y, whose_turn)) {
       move (x, y, whose_turn);
     } else {
@@ -419,21 +397,19 @@ load_pixmaps (void)
 {
   GdkPixbuf *image;
   GError *error = NULL;
-  gchar *tmp;
   gchar *fname;
+  const char *dname;
 
   g_return_if_fail (tile_set != NULL && tile_set[0] != '0');
 
-  tmp = g_build_filename ("iagno", tile_set, NULL);
-  fname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-				     tmp, FALSE, NULL);
-  g_free (tmp);
+  dname = games_runtime_get_directory (GAMES_RUNTIME_GAME_PIXMAP_DIRECTORY);
 
-  if (!g_file_test (fname, G_FILE_TEST_EXISTS)) {
+  fname = g_build_filename (dname, tile_set, NULL);
+
+  /* fall back to default tileset "classic.png" if tile_set not found*/
+  if (!g_file_test (fname, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
     g_free (fname);
-    fname = gnome_program_locate_file (NULL,
-				       GNOME_FILE_DOMAIN_APP_PIXMAP,
-				       "iagno/classic.png", FALSE, NULL);
+    fname = g_build_filename (dname, "classic.png", NULL);
   }
 
   if (!g_file_test (fname, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
@@ -457,14 +433,40 @@ load_pixmaps (void)
   g_free (fname);
 }
 
+void
+set_animation_speed (gint speed)
+{
+    flip_animation_speed = speed;
+    if (flip_pixmaps_id) {
+        g_source_remove (flip_pixmaps_id);
+        flip_pixmaps_id = g_timeout_add (flip_animation_speed, flip_pixmaps, NULL);
+    }
+}
+
+void
+start_animation (void)
+{
+    if (flip_pixmaps_id == 0)
+        flip_pixmaps_id = g_timeout_add (flip_animation_speed, flip_pixmaps, NULL);
+    tiles_to_flip = 1;
+}
+
+void
+stop_animation (void)
+{
+    tiles_to_flip = 0;
+}
+
 gint
 flip_pixmaps (gpointer data)
 {
   guint i, j;
   guint flipped_tiles = 0;
 
-  if (!tiles_to_flip)
-    return TRUE;
+  if (!tiles_to_flip) {
+      flip_pixmaps_id = 0;
+      return FALSE;
+  }
 
   for (i = 0; i < 8; i++)
     for (j = 0; j < 8; j++) {
@@ -530,7 +532,7 @@ flip_pixmaps (gpointer data)
     }
 
   if (!flipped_tiles)
-    tiles_to_flip = 0;
+    stop_animation ();
 
   return TRUE;
 }
@@ -607,88 +609,10 @@ init_new_game (void)
 
   whose_turn = BLACK_TURN;
   gui_status ();
-  gtk_widget_show (mainmenu[0].widget);
-  gtk_widget_show (mainmenu[1].widget);
 
   check_computer_players ();
 }
 
-void
-create_window (void)
-{
-  GtkWidget *table;
-
-  window = gnome_app_new ("iagno", _(APP_NAME_LONG));
-
-  games_conf_add_window (GTK_WINDOW (window), NULL);
-
-  notebook = gtk_notebook_new ();
-  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
-
-  gtk_widget_realize (window);
-  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
-  g_signal_connect (G_OBJECT (window), "delete_event",
-		    G_CALLBACK (quit_game_cb), NULL);
-
-  gnome_app_create_menus (GNOME_APP (window), mainmenu);
-
-  drawing_area = gtk_drawing_area_new ();
-
-  gtk_widget_pop_colormap ();
-
-  gnome_app_set_contents (GNOME_APP (window), notebook);
-  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), drawing_area, NULL);
-  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), MAIN_PAGE);
-
-  gtk_widget_set_size_request (GTK_WIDGET (drawing_area),
-			       BOARDWIDTH, BOARDHEIGHT);
-  g_signal_connect (G_OBJECT (drawing_area), "expose_event",
-		    G_CALLBACK (expose_event), NULL);
-  g_signal_connect (G_OBJECT (window), "configure_event",
-		    G_CALLBACK (configure_event), NULL);
-  g_signal_connect (G_OBJECT (drawing_area), "button_press_event",
-		    G_CALLBACK (button_press_event), NULL);
-  gtk_widget_set_events (drawing_area,
-			 GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
-  /* We do our own double-buffering. */
-  gtk_widget_set_double_buffered (drawing_area, FALSE);
-
-  gtk_widget_show (drawing_area);
-
-  appbar = GNOME_APPBAR (gnome_appbar_new (FALSE, TRUE, FALSE));
-  gnome_app_set_statusbar (GNOME_APP (window), GTK_WIDGET (appbar));
-  gnome_app_install_menu_hints (GNOME_APP (window), mainmenu);
-
-  table = gtk_table_new (1, 8, FALSE);
-
-  black_score = gtk_label_new (_("Dark:"));
-  gtk_widget_show (black_score);
-
-  gtk_table_attach (GTK_TABLE (table), black_score, 1, 2, 0, 1, 0, 0, 3, 1);
-
-  black_score = gtk_label_new ("00");
-  gtk_widget_show (black_score);
-
-  gtk_table_attach (GTK_TABLE (table), black_score, 2, 3, 0, 1, 0, 0, 3, 1);
-
-  white_score = gtk_label_new (_("Light:"));
-  gtk_widget_show (white_score);
-
-  gtk_table_attach (GTK_TABLE (table), white_score, 4, 5, 0, 1, 0, 0, 3, 1);
-
-  white_score = gtk_label_new ("00");
-  gtk_widget_show (white_score);
-
-  gtk_table_attach (GTK_TABLE (table), white_score, 5, 6, 0, 1, 0, 0, 3, 1);
-  undo_set_sensitive (FALSE);
-
-  gtk_widget_show (table);
-
-  gtk_box_pack_start (GTK_BOX (appbar), table, FALSE, TRUE, 0);
-
-  gnome_appbar_set_status (GNOME_APPBAR (appbar), _("Welcome to Iagno!"));
-}
-
 void
 gui_status (void)
 {
@@ -701,28 +625,20 @@ gui_status (void)
   undo_set_sensitive (move_count > 0);
 
   if (ggz_network_mode) {
-    gtk_widget_hide (game_menu[0].widget);
-    gtk_widget_hide (game_menu[1].widget);
-    gtk_widget_hide (game_menu[2].widget);
-    gtk_widget_hide (game_menu[3].widget);
-    gtk_widget_show (game_menu[4].widget);
-    gtk_widget_show (game_menu[5].widget);
-    gtk_widget_show (game_menu[7].widget);
+    gtk_action_set_sensitive(new_game_action, FALSE);
+    gtk_action_set_sensitive(new_network_action, FALSE);
+    gtk_action_set_sensitive(leave_network_action, FALSE);
+    gtk_action_set_sensitive(player_list_action, FALSE);
   } else {
-    gtk_widget_show (game_menu[0].widget);
-    gtk_widget_show (game_menu[1].widget);
-    gtk_widget_show (game_menu[2].widget);
-    gtk_widget_show (game_menu[3].widget);
-    gtk_widget_hide (game_menu[4].widget);
-    gtk_widget_hide (game_menu[5].widget);
-    gtk_widget_hide (game_menu[7].widget);
+    gtk_action_set_sensitive(new_game_action, TRUE);
+    gtk_action_set_sensitive(new_network_action, TRUE);
+    gtk_action_set_sensitive(leave_network_action, TRUE);
+    gtk_action_set_sensitive(player_list_action, TRUE);
   }
 
-  gtk_widget_set_sensitive (settings_menu[0].widget, !ggz_network_mode);
-
 #ifndef GGZ_CLIENT
-  gtk_widget_set_sensitive (game_menu[1].widget, FALSE);
-#endif
+    gtk_action_set_sensitive(new_network_action, FALSE);
+#endif /* GGZ_CLIENT */
 
   if (ggz_network_mode) {
     if (whose_turn == player_id && whose_turn == BLACK_TURN) {
@@ -744,15 +660,13 @@ gui_status (void)
     }
 
   }
-
-
 }
 
 void
 gui_message (gchar * message)
 {
-  gnome_appbar_pop (GNOME_APPBAR (appbar));
-  gnome_appbar_push (GNOME_APPBAR (appbar), message);
+  gtk_statusbar_pop (GTK_STATUSBAR(statusbar), statusbar_id);
+  gtk_statusbar_push (GTK_STATUSBAR(statusbar), statusbar_id, message);
 }
 
 guint
@@ -840,26 +754,26 @@ set_bg_color (void)
   g_object_unref (tmpimage);
 }
 
+#ifdef WITH_SMCLIENT
 static int
-save_state (GnomeClient * client, gint phase, GnomeRestartStyle save_style,
-	    gint shutdown, GnomeInteractStyle interact_style,
-	    gint fast, gpointer client_data)
+save_state_cb (EggSMClient *client,
+	    GKeyFile* keyfile,
+	    gpointer client_data)
 {
-  char *argv[20];
-  int i;
+  gchar *argv[20];
+  gint argc;
   gint xpos, ypos;
 
   gdk_window_get_origin (window->window, &xpos, &ypos);
 
-  i = 0;
-  argv[i++] = (char *) client_data;
-  argv[i++] = "-x";
-  argv[i++] = g_strdup_printf ("%d", xpos);
-  argv[i++] = "-y";
-  argv[i++] = g_strdup_printf ("%d", ypos);
+  argc = 0;
+  argv[argc++] = g_get_prgname ();
+  argv[argc++] = "-x";
+  argv[argc++] = g_strdup_printf ("%d", xpos);
+  argv[argc++] = "-y";
+  argv[argc++] = g_strdup_printf ("%d", ypos);
 
-  gnome_client_set_restart_command (client, i, argv);
-  gnome_client_set_clone_command (client, 0, NULL);
+  egg_sm_client_set_restart_command (client, argc, (const char **) argv);
 
   g_free (argv[2]);
   g_free (argv[4]);
@@ -867,59 +781,271 @@ save_state (GnomeClient * client, gint phase, GnomeRestartStyle save_style,
   return TRUE;
 }
 
+static gint
+quit_cb (EggSMClient *client,
+         gpointer client_data)
+{
+  gtk_main_quit ();
+
+  return FALSE;
+}
+
+#endif /* WITH_SMCLIENT */
+
+static void
+help_cb (GtkAction * action, gpointer data)
+{
+  games_help_display (window, APP_NAME, NULL);
+}
+
+static const GtkActionEntry actions[] = {
+  {"GameMenu", NULL, N_("_Game")},
+  {"SettingsMenu", NULL, N_("_Settings")},
+  {"HelpMenu", NULL, N_("_Help")},
+  {"NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, 
+   G_CALLBACK (new_game_cb)},
+#ifdef GGZ_CLIENT
+  {"NewNetworkGame", GAMES_STOCK_NETWORK_GAME, NULL, NULL, NULL,
+   G_CALLBACK (new_network_game_cb)},
+#else
+  {"NewNetworkGame", GAMES_STOCK_NETWORK_GAME, NULL, NULL, NULL, NULL},
+#endif
+  {"LeaveNetworkGame", GAMES_STOCK_NETWORK_LEAVE, NULL, NULL, NULL,
+   G_CALLBACK (on_network_leave)},
+  {"PlayerList", GAMES_STOCK_PLAYER_LIST, NULL, NULL, NULL,
+   G_CALLBACK (on_player_list)},
+  {"UndoMove", GAMES_STOCK_UNDO_MOVE, NULL, NULL, NULL,
+   G_CALLBACK (undo_move_cb)},
+  {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_game_cb)},
+  {"Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL,
+   G_CALLBACK (properties_cb)},
+  {"Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL,
+   G_CALLBACK (help_cb)},
+  {"About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_cb)}
+};
+
+static const char ui_description[] =
+  "<ui>"
+  "  <menubar name='MainMenu'>"
+  "    <menu action='GameMenu'>"
+  "      <menuitem action='NewGame'/>"
+  "      <menuitem action='NewNetworkGame'/>"
+  "      <separator/>"
+  "      <menuitem action='UndoMove'/>"
+  "      <separator/>"
+  "      <menuitem action='Quit'/>"
+  "    </menu>"
+  "    <menu action='SettingsMenu'>"
+  "      <menuitem action='Preferences'/>"
+  "    </menu>"
+  "    <menu action='HelpMenu'>"
+  "      <menuitem action='Contents'/>"
+  "      <menuitem action='About'/>"
+  "    </menu>"
+  "  </menubar>"
+  "</ui>";
+
+
+static void
+create_menus (GtkUIManager * ui_manager)
+{
+  GtkActionGroup *action_group;
+
+
+  action_group = gtk_action_group_new ("group");
+
+  gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+  gtk_action_group_add_actions (action_group, actions, G_N_ELEMENTS (actions),
+				window);
+
+  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+  gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
+
+  gtk_window_add_accel_group (GTK_WINDOW (window),
+			      gtk_ui_manager_get_accel_group (ui_manager));
+
+  new_game_action = gtk_action_group_get_action (action_group, "NewGame");
+  new_network_action = gtk_action_group_get_action (action_group,
+						    "NewNetworkGame");
+#ifndef GGZ_CLIENT
+  gtk_action_set_sensitive (new_network_action, FALSE);
+#endif /* GGZ_CLIENT */
+  leave_network_action =
+    gtk_action_group_get_action (action_group, "LeaveNetworkGame");
+  player_list_action =
+    gtk_action_group_get_action (action_group, "PlayerList");
+  undo_action = gtk_action_group_get_action (action_group, "UndoMove");
+}
+
+void
+create_window (void)
+{
+  GtkWidget *table;
+  GtkUIManager *ui_manager;
+  GtkWidget *menubar;
+  GtkWidget *vbox;
+
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (window), _(APP_NAME_LONG));
+
+  games_conf_add_window (GTK_WINDOW (window), NULL);
+
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (window), vbox);
+
+  ui_manager = gtk_ui_manager_new ();
+  create_menus (ui_manager);
+  menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
+  gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
+
+  notebook = gtk_notebook_new ();
+  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
+
+  g_signal_connect (G_OBJECT (window), "delete_event",
+		    G_CALLBACK (quit_game_cb), NULL);
+
+  drawing_area = gtk_drawing_area_new ();
+
+  gtk_widget_pop_colormap ();
+
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), drawing_area, NULL);
+  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), MAIN_PAGE);
+  gtk_box_pack_start (GTK_BOX (vbox), notebook, FALSE, FALSE, 0);
+
+  gtk_widget_set_size_request (GTK_WIDGET (drawing_area),
+			       BOARDWIDTH, BOARDHEIGHT);
+
+  g_signal_connect (G_OBJECT (drawing_area), "expose_event",
+		    G_CALLBACK (expose_event), NULL);
+  g_signal_connect (G_OBJECT (window), "configure_event",
+		    G_CALLBACK (configure_event), NULL);
+  g_signal_connect (G_OBJECT (drawing_area), "button_press_event",
+		    G_CALLBACK (button_press_event), NULL);
+
+  gtk_widget_set_events (drawing_area,
+			 GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
+  /* We do our own double-buffering. */
+  gtk_widget_set_double_buffered (drawing_area, FALSE);
+
+  statusbar = gtk_statusbar_new ();
+  gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);
+
+  table = gtk_table_new (1, 8, FALSE);
+
+  black_score = gtk_label_new (_("Dark:"));
+  gtk_widget_show (black_score);
+
+  gtk_table_attach (GTK_TABLE (table), black_score, 1, 2, 0, 1, 0, 0, 3, 1);
+
+  black_score = gtk_label_new ("00");
+  gtk_widget_show (black_score);
+
+  gtk_table_attach (GTK_TABLE (table), black_score, 2, 3, 0, 1, 0, 0, 3, 1);
+
+  white_score = gtk_label_new (_("Light:"));
+  gtk_widget_show (white_score);
+
+  gtk_table_attach (GTK_TABLE (table), white_score, 4, 5, 0, 1, 0, 0, 3, 1);
+
+  white_score = gtk_label_new ("00");
+  gtk_widget_show (white_score);
+
+  gtk_table_attach (GTK_TABLE (table), white_score, 5, 6, 0, 1, 0, 0, 3, 1);
+  undo_set_sensitive (FALSE);
+
+  gtk_widget_show (vbox);
+  gtk_widget_show (drawing_area);
+  gtk_widget_show (notebook);
+  gtk_widget_show (table);
+  gtk_widget_show (statusbar);
+
+  gtk_box_pack_start (GTK_BOX (statusbar), table, FALSE, TRUE, 0);
+
+  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
+
+  statusbar_id = gtk_statusbar_get_context_id (GTK_STATUSBAR(statusbar),
+                                               "iagno");
+  gtk_statusbar_push (GTK_STATUSBAR(statusbar), statusbar_id, 
+                      _("Welcome to Iagno!"));
+}
+
 int
 main (int argc, char **argv)
 {
-  GnomeClient *client;
-  GnomeProgram *program;
   GOptionContext *context;
+  gboolean retval;
+  GError *error = NULL;
+#ifdef WITH_SMCLIENT
+  EggSMClient *sm_client;
+#endif /* WITH_SMCLIENT */
 
-  bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS)
+  /* If we're going to use gnome-vfs, we need to init threads before
+   * calling any glib functions.
+   */
+  g_thread_init (NULL);
+#endif
+
+  if (!games_runtime_init ("iagno"))
+    return 1;
+
+  bindtextdomain (GETTEXT_PACKAGE, games_runtime_get_directory (GAMES_RUNTIME_LOCALE_DIRECTORY));
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  g_thread_init (NULL);
-
   context = g_option_context_new (NULL);
+#if GLIB_CHECK_VERSION (2, 12, 0)
+  g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
+#endif
+  g_option_context_add_group (context, gtk_get_option_group (TRUE));
+#ifdef WITH_SMCLIENT
+  g_option_context_add_group (context, egg_sm_client_get_option_group ());
+#endif /* WITH_SMCLIENT */
   g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
   games_sound_add_option_group (context);
 
-  program = gnome_program_init ("iagno", VERSION,
-				LIBGNOMEUI_MODULE,
-				argc, argv,
-				GNOME_PARAM_GOPTION_CONTEXT, context,
-				GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+  retval = g_option_context_parse (context, &argc, &argv, &error);
+  g_option_context_free (context);
+  if (!retval) {
+    g_print ("%s", error->message);
+    g_error_free (error);
+    exit (1);
+  }
 
-  gtk_window_set_default_icon_name ("gnome-iagno");
+  g_set_application_name (_(APP_NAME_LONG));
 
   games_conf_initialise (APP_NAME);
 
-  client = gnome_master_client ();
+  games_stock_init ();
 
-  g_signal_connect (G_OBJECT (client), "save_yourself",
-		    G_CALLBACK (save_state), argv[0]);
-  g_signal_connect (G_OBJECT (client), "die",
-		    G_CALLBACK (quit_game_cb), argv[0]);
+  gtk_window_set_default_icon_name ("gnome-iagno");
+
+#ifdef WITH_SMCLIENT
+  sm_client = egg_sm_client_get ();
+  g_signal_connect (sm_client, "save-state",
+		    G_CALLBACK (save_state_cb), NULL);
+  g_signal_connect (sm_client, "quit",
+                    G_CALLBACK (quit_cb), NULL);
+#endif /* WITH_SMCLIENT */
 
   create_window ();
 
+  gtk_widget_show (window);
+  buffer_pixmap = gdk_pixmap_new (drawing_area->window,
+				  BOARDWIDTH, BOARDHEIGHT, -1);
+
   load_properties ();
 
   load_pixmaps ();
 
-  gtk_widget_show (window);
-
 #ifdef GGZ_CLIENT
   network_init ();
-#endif
+#endif /* GGZ_CLIENT */
 
   if (session_xpos >= 0 && session_ypos >= 0) {
     gdk_window_move (window->window, session_xpos, session_ypos);
   }
 
-  buffer_pixmap = gdk_pixmap_new (drawing_area->window,
-				  BOARDWIDTH, BOARDHEIGHT, -1);
-
   set_bg_color ();
 
   init_new_game ();
@@ -928,7 +1054,7 @@ main (int argc, char **argv)
 
   games_conf_shutdown ();
 
-  g_object_unref (program);
+  games_runtime_shutdown ();
 
   return 0;
 }
