@@ -19,19 +19,16 @@
 
 /* Authors:   Callum McKenzie <callum@physics.otago.ac.nz> */
 
-#include <glib/gi18n.h>
-#include <gtk/gtk.h>
+#include <config.h>
 
 #include <math.h>
 
-#include "config.h"
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #include "games-scores-dialog.h"
 #include "games-scores-dialog-private.h"
 
-static   gpointer games_scores_dialog_parent_class;
-
-
 G_DEFINE_TYPE (GamesScoresDialog, games_scores_dialog, GTK_TYPE_DIALOG);
 
 static void
@@ -47,8 +44,7 @@ games_scores_dialog_finalize (GObject *o)
   if (dialog->_priv->catindices)
     g_hash_table_destroy (dialog->_priv->catindices);
 
-  if (G_OBJECT_CLASS (games_scores_dialog_parent_class)->finalize)
-    (*G_OBJECT_CLASS (games_scores_dialog_parent_class)->finalize) (o);
+  G_OBJECT_CLASS (games_scores_dialog_parent_class)->finalize (o);
 }
 
 static void
@@ -56,8 +52,6 @@ games_scores_dialog_class_init (GamesScoresDialogClass *klass)
 {
   g_type_class_add_private (klass, sizeof (GamesScoresDialogPrivate));
 
-  games_scores_dialog_parent_class = g_type_class_peek_parent (klass);
-
   G_OBJECT_CLASS (klass)->finalize = games_scores_dialog_finalize;
 }
 
@@ -160,7 +154,7 @@ static void games_scores_dialog_set_category (GamesScoresDialog *self,
  * Returns: a new widget
  *
  **/
-GtkWidget * games_scores_dialog_new (GamesScores *scores, const gchar *title)
+GtkWidget * games_scores_dialog_new (GtkWindow *parent_window, GamesScores *scores, const gchar *title)
 {
   GamesScoresDialog *dialog = GAMES_SCORES_DIALOG (g_object_new (GAMES_TYPE_SCORES_DIALOG, NULL));
 
@@ -169,6 +163,7 @@ GtkWidget * games_scores_dialog_new (GamesScores *scores, const gchar *title)
   dialog->_priv->preservehilight = FALSE;
 
   gtk_window_set_title (GTK_WINDOW (dialog), title);
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent_window));
 
   games_scores_category_foreach (scores, 
 				 (GamesScoresCategoryForeachFunc) games_scores_dialog_load_categories, 
@@ -225,7 +220,7 @@ static gboolean games_scores_dialog_set_edit (GamesScoresDialog *self)
 			      self->_priv->namecolumn, TRUE);
   g_signal_handler_unblock (self->_priv->treeview, 
 			      self->_priv->cursor_handler_id); 
-  g_free (path);
+  gtk_tree_path_free (path);
 
   return FALSE;
 }
@@ -355,8 +350,8 @@ void games_scores_dialog_set_category_description (GamesScoresDialog *self,
   gchar *lstr;
 
   lstr = g_strdup_printf ("<b>%s</b>", description);
-  gtk_label_set_text (GTK_LABEL (self->_priv->label), lstr);
-  gtk_label_set_use_markup (GTK_LABEL (self->_priv->label), TRUE);
+  gtk_label_set_markup (GTK_LABEL (self->_priv->label), lstr);
+  gtk_label_set_use_underline (GTK_LABEL (self->_priv->label), TRUE);
   g_free(lstr);
 }
 
@@ -507,13 +502,15 @@ static void games_scores_dialog_init (GamesScoresDialog *self)
   self->_priv->catbar = gtk_hbox_new (FALSE, 12);
   gtk_box_pack_start (GTK_BOX (vbox), self->_priv->catbar, FALSE, FALSE, 0);
 
-  self->_priv->label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (self->_priv->catbar), self->_priv->label, 
+  self->_priv->label = gtk_label_new (NULL);
+  gtk_label_set_use_markup (GTK_LABEL (self->_priv->label), TRUE);
+  gtk_box_pack_start (GTK_BOX (self->_priv->catbar), self->_priv->label,
 			FALSE, FALSE, 0);	
  
   self->_priv->combo = gtk_combo_box_new_text ();
   gtk_box_pack_start (GTK_BOX (self->_priv->catbar), 
 			self->_priv->combo, TRUE, TRUE, 0);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (self->_priv->label), self->_priv->combo);
 
   g_signal_connect (G_OBJECT (self->_priv->combo), "changed", 
 		      G_CALLBACK (games_scores_dialog_change_category), self);
